@@ -1,7 +1,7 @@
 /* display.c -- How to display Info windows.
-   $Id: display.c,v 1.14 2008/05/12 23:34:09 gray Exp $
+   $Id: display.c 5338 2013-08-22 17:58:30Z karl $
 
-   Copyright (C) 1993, 1997, 2003, 2004, 2006, 2007
+   Copyright 1993, 1997, 2003, 2004, 2006, 2007, 2008, 2012, 2013
    Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -17,23 +17,20 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-   Originally written by Brian Fox (bfox@ai.mit.edu). */
+   Originally written by Brian Fox. */
 
 #include "info.h"
 #include "display.h"
+#include "tag.h"
 
 extern int info_any_buffered_input_p (void); /* Found in session.c. */
 
 static void free_display (DISPLAY_LINE **display);
 static DISPLAY_LINE **make_display (int width, int height);
 
-void handle_tag (char *tag);
-void handle_tag_start (char *tag);
-void handle_tag_end (char *tag);
-
 /* An array of display lines which tell us what is currently visible on
    the display.  */
-DISPLAY_LINE **the_display = (DISPLAY_LINE **)NULL;
+DISPLAY_LINE **the_display = NULL;
 
 /* Non-zero means do no output. */
 int display_inhibited = 0;
@@ -90,332 +87,6 @@ display_update_display (WINDOW *window)
   /* Always update the echo area. */
   display_update_one_window (the_echo_area);
 }
-
-void
-handle_tag_start (char *tag)
-{
-  /* TODO really handle this tag.  */
-  return;
-}
-
-void
-handle_tag_end (char *tag)
-{
-  /* TODO really handle this tag.  */
-  return;
-}
-
-void
-handle_tag (char *tag)
-{
-    if (tag[0] == '/')
-      {
-	tag++;
-	handle_tag_end (tag);
-      }
-    else
-      handle_tag_start (tag);
-}
-
-
-#define ITER_SETBYTES(iter,n) ((iter).cur.bytes = n)
-#define ITER_LIMIT(iter) ((iter).limit - (iter).cur.ptr)
-
-/* If ITER points to an ANSI escape sequence, process it, set PLEN to its
-   length in bytes, and return 1.
-   Otherwise, return 0.
- */
-static int
-ansi_escape (mbi_iterator_t iter, size_t *plen)
-{
-  if (raw_escapes_p && *mbi_cur_ptr (iter) == '\033' && mbi_avail (iter))
-    {
-      mbi_advance (iter);
-      if (*mbi_cur_ptr (iter) == '[' &&  mbi_avail (iter))
-	{
-	  ITER_SETBYTES (iter, 1);
-	  mbi_advance (iter);
-	  if (isdigit (*mbi_cur_ptr (iter)) && mbi_avail (iter))
-	    {	
-	      ITER_SETBYTES (iter, 1);
-	      mbi_advance (iter);
-	      if (*mbi_cur_ptr (iter) == 'm')
-		{
-		  *plen = 4;
-		  return 1;
-		}
-	      else if (isdigit (*mbi_cur_ptr (iter)) && mbi_avail (iter))
-		{
-		  ITER_SETBYTES (iter, 1);
-		  mbi_advance (iter);
-		  if (*mbi_cur_ptr (iter) == 'm')
-		    {
-		      *plen = 5;
-		      return 1;
-		    }
-		}
-	    }
-	}
-    }
-		
-  return 0;
-}
-
-/* If ITER points to an info tag, process it, set PLEN to its
-   length in bytes, and return 1.
-   Otherwise, return 0.
-
-   Collected tag is processed if HANDLE!=0.
-*/
-static int
-info_tag (mbi_iterator_t iter, int handle, size_t *plen)
-{
-  if (*mbi_cur_ptr (iter) == '\0' && mbi_avail (iter))
-    {
-      mbi_advance (iter);
-      if (*mbi_cur_ptr (iter) == '\b' && mbi_avail (iter))
-	{
-	  mbi_advance (iter);
-	  if (*mbi_cur_ptr (iter) == '[' && mbi_avail (iter))
-	    {
-	      const char *ptr, *end;
-	      mbi_advance (iter);
-	      ptr = mbi_cur_ptr (iter);
-	      end = memmem (ptr, ITER_LIMIT (iter), "\0\b]", 3);
-	      if (end)
-		{
-		  size_t len = end - ptr;
-
-		  if (handle)
-		    {
-		      char *elt = xmalloc (len + 1);
-		      memcpy (elt, ptr, len);
-		      elt[len] = 0;
-		      handle_tag (elt);
-		      free (elt);
-		    }
-		  *plen = len + 6;
-		  return 1;
-		}
-	    }
-	}
-    }
-
-  return 0;
-}
-
-/* Process contents of the current node from WIN, beginning from START, using
-   callback function FUN.
-
-   FUN is called for every line collected from the node. Its arguments:
-
-     int (*fun) (void *closure, size_t line_no,
-                  const char *src_line, char *prt_line,
-		  size_t prt_bytes, size_t prt_chars)
-
-     closure  -- An opaque pointer passed as 5th parameter to process_node_text;
-     line_no  -- Number of processed line (starts from 0);
-     src_line -- Pointer to the source line (unmodified);
-     prt_line -- Collected line contents, ready for output;
-     prt_bytes -- Number of bytes in prt_line;
-     prt_chars -- Number of characters in prt_line.
-
-   If FUN returns non zero, process_node_text stops processing and returns
-   immediately.
-
-   If DO_TAGS is not zero, process info tags, otherwise ignore them.
-
-   Return value: number of lines processed.
-*/
-   
-size_t
-process_node_text (WINDOW *win, char *start,
-		   int do_tags,
-		   int (*fun) (void *, size_t, const char *, char *, size_t, size_t),
-		   void *closure)
-{
-  char *printed_line;      /* Buffer for a printed line. */
-  size_t pl_count = 0;     /* Number of *characters* written to PRINTED_LINE */
-  size_t pl_index = 0;     /* Index into PRINTED_LINE. */
-  size_t in_index = 0;
-  size_t line_index = 0;   /* Number of lines done so far. */
-  size_t allocated_win_width;
-  mbi_iterator_t iter;
-  
-  /* Print each line in the window into our local buffer, and then
-     check the contents of that buffer against the display.  If they
-     differ, update the display. */
-  allocated_win_width = win->width + 1;
-  printed_line = xmalloc (allocated_win_width);
-
-  for (mbi_init (iter, start, 
-		 win->node->contents + win->node->nodelen - start),
-	 pl_count = 0;
-       mbi_avail (iter);
-       mbi_advance (iter))
-    {
-      const char *carried_over_ptr;
-      size_t carried_over_len, carried_over_count;
-      const char *cur_ptr = mbi_cur_ptr (iter);
-      int cur_len = mb_len (mbi_cur (iter));
-      int replen;
-      int delim = 0;
-      int rc;
-
-      if (mb_isprint (mbi_cur (iter)))
-	{
-	  replen = 1;
-	}
-      else if (cur_len == 1)
-	{
-          if (*cur_ptr == '\r' || *cur_ptr == '\n')
-            {
-              replen = win->width - pl_count;
-	      delim = 1;
-            }
-	  else if (ansi_escape (iter, &cur_len))
-	    {
-	      replen = 1;
-	      ITER_SETBYTES (iter, cur_len);
-	    }
-	  else if (info_tag (iter, do_tags, &cur_len)) 
-	    {
-	      ITER_SETBYTES (iter, cur_len);
-	      continue;
-	    }
-	  else
-	    {
-	      if (*cur_ptr == '\t')
-		delim = 1;
-              cur_ptr = printed_representation (cur_ptr, cur_len, pl_count,
-						&cur_len);
-	      replen = cur_len;
-            }
-        }
-      else
-	{
-	  /* FIXME: I'm not sure it's the best way to deal with unprintable
-	     multibyte characters */
-	  cur_ptr = printed_representation (cur_ptr, cur_len, pl_count,
-					    &cur_len);
-	  replen = cur_len;
-	}
-
-      /* Ensure there is enough space in the buffer */
-      while (pl_index + cur_len + 2 > allocated_win_width - 1)
-	printed_line = x2realloc (printed_line, &allocated_win_width);
-
-      /* If this character can be printed without passing the width of
-         the line, then stuff it into the line. */
-      if (pl_count + replen < win->width)
-        {
-	  int i;
-	  
-	  for (i = 0; i < cur_len; i++)
-	    printed_line[pl_index++] = cur_ptr[i];
-	  pl_count += replen;
-	  in_index += mb_len (mbi_cur (iter));
-        }
-      else
-	{
-          /* If this character cannot be printed in this line, we have
-             found the end of this line as it would appear on the screen.
-             Carefully print the end of the line, and then compare. */
-          if (delim)
-            {
-              printed_line[pl_index] = '\0';
-              carried_over_ptr = (char *)NULL;
-            }
-	  else
-	    {
-              /* The printed representation of this character extends into
-                 the next line. */
-
-	      carried_over_count = replen;
-	      if (replen == 1)
-		{
-		  /* It is a single (possibly multibyte) character */
-		  /* FIXME? */
-		  carried_over_ptr = cur_ptr;
-		  carried_over_len = cur_len;
-		}
-	      else
-		{
-		  int i;
-		  
-		  /* Remember the offset of the last character printed out of
-		     REP so that we can carry the character over to the next
-		     line. */
-		  for (i = 0; pl_count < (win->width - 1);
-		       pl_count++)
-		    printed_line[pl_index++] = cur_ptr[i++];
-
-		  carried_over_ptr = cur_ptr + i;
-		  carried_over_len = cur_len;
-		}
-
-              /* If printing the last character in this window couldn't
-                 possibly cause the screen to scroll, place a backslash
-                 in the rightmost column. */
-              if (1 + line_index + win->first_row < the_screen->height)
-                {
-                  if (win->flags & W_NoWrap)
-                    printed_line[pl_index++] = '$';
-                  else
-                    printed_line[pl_index++] = '\\';
-		  pl_count++;
-                }
-              printed_line[pl_index] = '\0';
-            }
-
-	  rc = fun (closure, line_index, mbi_cur_ptr (iter) - in_index,
-		    printed_line, pl_index, pl_count);
-
-          ++line_index;
-
-	  /* Reset all data to the start of the line. */
-	  pl_index = 0;
-	  pl_count = 0;
-	  in_index = 0;
-
-	  if (rc)
-	    break;
-	  
-          /* If there are bytes carried over, stuff them
-             into the buffer now. */
-          if (carried_over_ptr)
-	    {
-	      for (; carried_over_len;
-		   carried_over_len--, carried_over_ptr++, pl_index++)
-		printed_line[pl_index] = *carried_over_ptr;
-	      pl_count += carried_over_count;
-	    }
-	
-          /* If this window has chosen not to wrap lines, skip to the end
-             of the physical line in the buffer, and start a new line here. */
-          if (pl_index && win->flags & W_NoWrap)
-            {
-	      for (; mbi_avail (iter); mbi_advance (iter))
-		if (mb_len (mbi_cur (iter)) == 1
-		    && *mbi_cur_ptr (iter) == '\n')
-		  break;
-
-	      pl_index = 0;
-	      pl_count = 0;
-	      in_index = 0;
-	      printed_line[0] = 0;
-	    }
-	}
-    }
-
-  if (pl_count)
-    fun (closure, line_index, mbi_cur_ptr (iter) - in_index,
-	 printed_line, pl_index, pl_count);
-
-  free (printed_line);
-  return line_index;
-}
-
 
 struct display_node_closure {
   WINDOW *win;
@@ -442,14 +113,14 @@ find_diff (const char *a, size_t alen, const char *b, size_t blen, int *ppos)
 }
 
 int
-display_node_text(void *closure, size_t line_index,
+display_node_text(void *closure, size_t pline_index, size_t lline_index,
 		  const char *src_line,
 		  char *printed_line, size_t pl_index, size_t pl_count)
 {
   struct display_node_closure *dn = closure;
   WINDOW *win = dn->win;
   DISPLAY_LINE **display = dn->display;
-  DISPLAY_LINE *entry = display[win->first_row + line_index];
+  DISPLAY_LINE *entry = display[win->first_row + pline_index];
 
   /* We have the exact line as it should appear on the screen.
      Check to see if this line matches the one already appearing
@@ -468,7 +139,7 @@ display_node_text(void *closure, size_t line_index,
 	  /* Need to erase the line if it has escape sequences.  */
 	  || (raw_escapes_p && mbschr (entry->text, '\033') != 0))
 	{
-	  terminal_goto_xy (0, win->first_row + line_index);
+	  terminal_goto_xy (0, win->first_row + pline_index);
 	  terminal_clear_to_eol ();
 	  entry->inverse = 0;
 	  entry->text[0] = '\0';
@@ -483,7 +154,7 @@ display_node_text(void *closure, size_t line_index,
       if (i != pl_count || pl_count != entry->textlen)
 	{
 	  /* Move to the proper point on the terminal. */
-	  terminal_goto_xy (i, win->first_row + line_index);
+	  terminal_goto_xy (i, win->first_row + pline_index);
 	  /* If there is any text to print, print it. */
 	  if (i != pl_count)
 	    terminal_put_text (printed_line + i);
@@ -525,7 +196,7 @@ display_node_text(void *closure, size_t line_index,
       return 1;
     }
 
-  if (line_index + 1 == win->height)
+  if (pline_index + 1 == win->height)
     return 1;
 
   return 0;
@@ -564,14 +235,15 @@ display_update_one_window (WINDOW *win)
       dnc.win = win;
       dnc.display = the_display;
       
-      line_index = process_node_text (win, win->line_starts[win->pagetop],
+      line_index = process_node_text (win,
+				      win->line_starts[win->pagetop],
 				      1,
 				      display_node_text,
 				      &dnc);
       if (display_was_interrupted_p)
 	return;
     }
-  
+
   /* We have reached the end of the node or the end of the window.  If it
      is the end of the node, then clear the lines of the window from here
      to the end of the window. */
@@ -610,7 +282,6 @@ display_update_one_window (WINDOW *win)
           strcpy (display[line_index]->text, win->modeline);
           display[line_index]->inverse = 1;
           display[line_index]->textlen = strlen (win->modeline);
-          fflush (stdout);
         }
     }
 
@@ -653,7 +324,7 @@ display_scroll_display (int start, int end, int amount)
       last = end + amount;
 
       /* Shift the lines to scroll right into place. */
-      for (i = 0; i < (end - start); i++)
+      for (i = 1; i <= (end - start); i++)
         {
           temp = the_display[last - i];
           the_display[last - i] = the_display[end - i];
@@ -669,8 +340,7 @@ display_scroll_display (int start, int end, int amount)
           the_display[i]->inverse = 0;
         }
     }
-
-  if (amount < 0)
+  else
     {
       last = start + amount;
       for (i = 0; i < (end - start); i++)
@@ -792,17 +462,17 @@ make_display (int width, int height)
   register int i;
   DISPLAY_LINE **display;
 
-  display = (DISPLAY_LINE **)xmalloc ((1 + height) * sizeof (DISPLAY_LINE *));
+  display = xmalloc ((1 + height) * sizeof (DISPLAY_LINE *));
 
   for (i = 0; i < height; i++)
     {
-      display[i] = (DISPLAY_LINE *)xmalloc (sizeof (DISPLAY_LINE));
-      display[i]->text = (char *)xmalloc (1 + width);
+      display[i] = xmalloc (sizeof (DISPLAY_LINE));
+      display[i]->text = xmalloc (1 + width);
       display[i]->textlen = 0;
       display[i]->inverse = 0;
     }
-  display[i] = (DISPLAY_LINE *)NULL;
-  return (display);
+  display[i] = NULL;
+  return display;
 }
 
 /* Free the storage allocated to DISPLAY. */
